<?php

declare(strict_types=1);

namespace Silex\Gateway;

use DateTime;
use PDO;
use Silex\Model\News;

class NewsGateway
{
    private const EXCERPT_LENGTH = 180;

    private PDO $pdo;

    public function __construct(PDO $pdo)
    {
        $this->pdo = $pdo;
    }

    /**
     * @return News[]
     */
    public function getPaginatedRecentNews(int $page = 1, int $limit = 10): array
    {
        $req = $this->pdo->prepare('SELECT title, LEFT(content, ' . self::EXCERPT_LENGTH . ') content, publication_date FROM news ORDER BY publication_date DESC LIMIT :limit OFFSET :offset;');
        $req->bindValue('limit', $limit, PDO::PARAM_INT);
        $req->bindValue('offset', ($page - 1) * $limit, PDO::PARAM_INT);
        if (!$req->execute()) {
            return [];
        }
        $news = [];
        while ($data = $req->fetch()) {
            $news[] = $this->createNews($data);
        }
        return $news;
    }

    public function getById(int $id): News 
    {
        $req = $this->pdo->prepare('SELECT * FROM news WHERE id_news=:id;');
        $req->bindValue(':id', $id, PDO::PARAM_INT);
        if (!$req->execute()) {
            return null;
        }
        $data = $req->fetch();
        $news = $this->createNews($data);
        return $news;
    }

    private function createNews(array $data): News
    {
        return new News($data['title'], $data['content'], DateTime::createFromFormat('Y-m-d H:i:s', $data['publication_date']));
    }
}
