<?php

declare(strict_types=1);

namespace Silex\Router;

use Silex\Http\HttpResponse;
use Silex\DI\DI;

class Router
{
    private string $url;

    private string $basePath = '';

    /**
     * @var Route[]
     */
    private array $routes = [];

    public function __construct(string $url)
    {
        $url = PathHelper::removeEverythingAfter($url, '?');
        $url = PathHelper::removeEverythingAfter($url, '#');
        $this->url = trim($url, '/');
    }

    public function setBasePath(string $basePath)
    {
        if (PathHelper::startsWith($this->url, $basePath)) {
            $this->url = trim(substr($this->url, strlen($basePath)), '/');
        }
        $this->basePath = $basePath;
    }

    public function get(string $path, callable $callable): self
    {
        return $this->addRoute(['GET'], $path, $callable);
    }

    public function post(string $path, callable $callable): self
    {
        return $this->addRoute(['GET'], $path, $callable);
    }

    public function match(string $path, callable $callable): self
    {
        return $this->addRoute(['GET', 'POST'], $path, $callable);
    }

    private function addRoute(array $methods, string $path, $callable): self
    {
        $route = new Route($path, $callable);
        foreach ($methods as $method) {
            $this->routes[$method][] = $route;
        }
        return $this;
    }

    public function url(string $url): string
    {
        return $this->basePath . '/' . $url;
    }

    public function run(DI $di): HttpResponse
    {
        if (!isset($this->routes[$_SERVER['REQUEST_METHOD']])) {
            throw new RouteNotFoundException('Unknown HTTP method');
        }
        if ($this->basePath === '' || PathHelper::startsWith($this->url, $this->basePath)) {
            foreach ($this->routes[$_SERVER['REQUEST_METHOD']] as $route) {
                if ($route->matches($this->url)) {
                    return $route->call($di);
                }
            }
        }
        throw new RouteNotFoundException('No matching routes');
    }
}
